/*
 * Copyright (c) 2023-2024. Helge Eichhorn and the LOX contributors
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, you can obtain one at https://mozilla.org/MPL/2.0/.
 */

//! generated holds trait implementations that have been generated by `lox-gen`, separating
//! generated code from manual implementations.
//!
//! Artifacts in its submodules are intended to be transparently re-exported.

pub mod barycenters;
pub mod minor;
pub mod planets;
pub mod satellites;
pub mod sun;
